# coding: utf-8
import os
from setuptools import setup
setup_pars = {
    "packages" : [
        'csst_dfs_commons',
        'csst_dfs_commons.convert',
        'csst_dfs_commons.logging',
        'csst_dfs_commons.models',
        'csst_dfs_commons.utils',
    ],
    "package_dir" : {
        'csst_dfs_commons' : 'csst_dfs_commons',
        'csst_dfs_commons.convert' : 'csst_dfs_commons/convert',
        'csst_dfs_commons.logging' : 'csst_dfs_commons/logging',
        'csst_dfs_commons.models' : 'csst_dfs_commons/models',
        'csst_dfs_commons.utils' : 'csst_dfs_commons/utils',
    },
}

def requirements():
    with open("requirements.txt", "r") as f:
        return [
            req.strip()
            for req in f.readlines()
            if not req.startswith("#") and req.__contains__("==")
        ]

def version():
    __version = {}
    version_path = os.path.join(os.path.dirname(__file__), "csst_dfs_commons", "version.py")
    with open(version_path, "r") as file:
        exec(file.read(), __version)
    return __version["__version__"]

setup(
    name="csst_dfs_commons",
    version=version(),
    description="common library for CSST Data Flow System (DFS)",
    long_description=open('README.md').read(),
    license="MIT",
    python_requires=">=3.7",
    install_requires=requirements(),
    zip_safe=False,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Physics",
        "Topic :: Scientific/Engineering :: Astronomy",

    ],
    include_package_data=False,
    project_urls={
        'Source': 'https://csst-tb.bao.ac.cn/code/csst-dfs/csst-dfs-api-local',
    },
    **setup_pars
)

